/**
 * GCodeRepRapPreferences.java
 * 
 * Preferences
 * @author Chris Meighan
 * 
 * This file is part of G-code for RepRap.
 * 
 * G-code for RepRap is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * G-code for RepRap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with G-code for RepRap.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.chrismeighan.repgcode;

import java.io.File;
import java.util.Properties;
import java.io.OutputStream;
import java.io.FileOutputStream;

/**
 * Preferences
 * @author chris
 *
 */
public class GCodeRepRapPreferences {
	private Properties p;
	private String path;
	
	public GCodeRepRapPreferences() {
		path = new String(System.getProperty("user.home") + File.separatorChar + ".repgcode");
		p = new Properties();
		load();
	}
	
	/**
	 * Read a property
	 * @param key
	 * @return
	 */
	public String getProperty(String key) {
		return p.getProperty(key);
	}
	
	/**
	 * Set a property
	 * @param key
	 * @param value
	 */
	public void setProperty(String key, String value) {
		p.setProperty(key, value);
	}
	
	/**
	 * Save preferences
	 */
	public void save() {
		try {
			// Make directory if necessary
			File pathFile = new File(path);
			if (!pathFile.isDirectory()) pathFile.mkdirs();
			
			// Save file
			File propsFile = new File(path + File.separatorChar + "repgcode.properties");
			OutputStream output = new FileOutputStream(propsFile);
			p.store(output,"G-code for RepRap properties");
		} catch ( Exception e ) {
		}
	}
	
	/** 
	 * Load preferences, saving defaults if file not present
	 */
	public void load() {
		File propsFile = new File(path + File.separatorChar + "repgcode.properties");
		if (propsFile.exists()) {
			try {
				p.load(propsFile.toURL().openStream());
			} catch ( Exception e) {
			}
		} else {
			
			p.setProperty("Port(name)", ((System.getProperty("os.name")
					.toLowerCase().substring(0, 7)).equals("windows")) ? "COM1"
					: "/dev/ttyS0");
			p.setProperty("XAxisAddress","2");
			p.setProperty("XAxisSteps","400");
			p.setProperty("XAxisMaxSpeed","200");
			p.setProperty("YAxisAddress","3");
			p.setProperty("YAxisSteps","400");
			p.setProperty("YAxisMaxSpeed","130");
			p.setProperty("ZAxisAddress","4");
			p.setProperty("ZAxisSteps","400");
			p.setProperty("ZAxisMaxSpeed","130");
			p.setProperty("Units","mm");
			p.setProperty("DistancePerRev","0.8");
			p.setProperty("DefaultFeedRate","30.0");
			p.setProperty("CurveSectionLength","1.0");
			p.setProperty("JogIncrement","1.0");
			save();
		}
	}
}