/**
 * SetupDialog.java
 * 
 * Preferences dialog
 * @author Chris Meighan
 * 
 * This file is part of G-code for RepRap.
 * 
 * G-code for RepRap is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * G-code for RepRap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with G-code for RepRap.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.chrismeighan.repgcode;

import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JFormattedTextField;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.BoxLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

public class SetupDialog extends JDialog implements ActionListener,
		DocumentListener {
	JFrame frame;
	Main parent;

	JTextField portNameField;
	JFormattedTextField xAddressField, xStepsField, xMaxSpeedField,
			yAddressField, yStepsField, yMaxSpeedField, zAddressField,
			zStepsField, zMaxSpeedField, distancePerRevField,
			defaultFeedRateField, curveSectionLengthField, jogIncrementField;
	JButton cancelButton, OKButton;
	JRadioButton mmRadioButton, inchesRadioButton;

	String units;

	Double distancePerRev, defaultFeedRate, curveSectionLength, jogIncrement;

	GCodeRepRapPreferences p;

	boolean watchFieldChanges = true;

	public SetupDialog(JFrame frame, Main parent) {
		super(frame, "Setup", true);
		this.parent = parent;

		NumberFormat integerFormat = NumberFormat.getIntegerInstance();
		NumberFormat decimalFormat = NumberFormat.getNumberInstance();

		p = new GCodeRepRapPreferences();
		p.load();

		JPanel options = new JPanel();

		// Preferences
		options.setLayout(new BoxLayout(options, BoxLayout.PAGE_AXIS));
		options.add(new JLabel("Port(name)"));
		portNameField = new JTextField(p.getProperty("Port(name)"), 20);
		options.add(portNameField);
		options.add(new JLabel("Address X"));
		xAddressField = new JFormattedTextField(integerFormat);
		xAddressField.setValue(Integer.valueOf(p.getProperty("XAxisAddress")));
		options.add(xAddressField);
		options.add(new JLabel("Motor steps X"));
		xStepsField = new JFormattedTextField(integerFormat);
		xStepsField.setValue(Integer.valueOf(p.getProperty("XAxisSteps")));
		options.add(xStepsField);
		options.add(new JLabel("Max speed X (rpm)"));
		xMaxSpeedField = new JFormattedTextField(integerFormat);
		xMaxSpeedField
				.setValue(Integer.valueOf(p.getProperty("XAxisMaxSpeed")));
		options.add(xMaxSpeedField);
		options.add(new JLabel("Address Y"));
		yAddressField = new JFormattedTextField(integerFormat);
		yAddressField.setValue(Integer.valueOf(p.getProperty("YAxisAddress")));
		options.add(yAddressField);
		options.add(new JLabel("Motor steps Y"));
		yStepsField = new JFormattedTextField(integerFormat);
		yStepsField.setValue(Integer.valueOf(p.getProperty("YAxisSteps")));
		options.add(yStepsField);
		options.add(new JLabel("Max speed Y (rpm)"));
		yMaxSpeedField = new JFormattedTextField(integerFormat);
		yMaxSpeedField
				.setValue(Integer.valueOf(p.getProperty("YAxisMaxSpeed")));
		options.add(yMaxSpeedField);
		options.add(new JLabel("Address Z"));
		zAddressField = new JFormattedTextField(integerFormat);
		zAddressField.setValue(Integer.valueOf(p.getProperty("ZAxisAddress")));
		options.add(zAddressField);
		options.add(new JLabel("Motor steps Z"));
		zStepsField = new JFormattedTextField(integerFormat);
		zStepsField.setValue(Integer.valueOf(p.getProperty("ZAxisSteps")));
		options.add(zStepsField);
		options.add(new JLabel("Max speed Z (rpm)"));
		zMaxSpeedField = new JFormattedTextField(integerFormat);
		zMaxSpeedField
				.setValue(Integer.valueOf(p.getProperty("ZAxisMaxSpeed")));
		options.add(zMaxSpeedField);
		options.add(new JLabel("Units"));
		units = p.getProperty("Units");
		mmRadioButton = new JRadioButton("mm");
		mmRadioButton.addActionListener(this);
		if (units.equals("mm"))
			mmRadioButton.setSelected(true);
		inchesRadioButton = new JRadioButton("inches");
		inchesRadioButton.addActionListener(this);
		if (units.equals("inches"))
			inchesRadioButton.setSelected(true);
		ButtonGroup unitRadioButtonGroup = new ButtonGroup();
		unitRadioButtonGroup.add(mmRadioButton);
		unitRadioButtonGroup.add(inchesRadioButton);
		options.add(mmRadioButton);
		options.add(inchesRadioButton);

		options.add(new JLabel(
				"Distance travelled per motor revolution (units/rev)"));
		distancePerRev = Double.valueOf(p.getProperty("DistancePerRev"));
		distancePerRevField = new JFormattedTextField(decimalFormat);
		distancePerRevField.setValue(distancePerRev);
		distancePerRevField.getDocument().addDocumentListener(this);
		distancePerRevField.getDocument().putProperty("name", "distancePerRev");
		options.add(distancePerRevField);
		options.add(new JLabel("Default feed rate (units/minute)"));
		defaultFeedRate = Double.valueOf(p.getProperty("DefaultFeedRate"));
		defaultFeedRateField = new JFormattedTextField(decimalFormat);
		defaultFeedRateField.setValue(defaultFeedRate);
		defaultFeedRateField.getDocument().addDocumentListener(this);
		defaultFeedRateField.getDocument().putProperty("name",
				"defaultFeedRate");
		options.add(defaultFeedRateField);
		options.add(new JLabel("Curve section length (units)"));
		curveSectionLength = Double
				.valueOf(p.getProperty("CurveSectionLength"));
		curveSectionLengthField = new JFormattedTextField(decimalFormat);
		curveSectionLengthField.setValue(curveSectionLength);
		curveSectionLengthField.getDocument().addDocumentListener(this);
		curveSectionLengthField.getDocument().putProperty("name",
				"curveSectionLength");
		options.add(curveSectionLengthField);
		options.add(new JLabel("Jog buttons increment (units)"));
		jogIncrement = Double.valueOf(p.getProperty("JogIncrement"));
		jogIncrementField = new JFormattedTextField(decimalFormat);
		jogIncrementField.setValue(jogIncrement);
		jogIncrementField.getDocument().addDocumentListener(this);
		jogIncrementField.getDocument().putProperty("name", "jogIncrement");
		options.add(jogIncrementField);

		// Action buttons
		JPanel actionButtons = new JPanel();
		actionButtons.setLayout(new BoxLayout(actionButtons,
				BoxLayout.LINE_AXIS));
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(this);
		actionButtons.add(cancelButton);
		OKButton = new JButton("OK");
		OKButton.addActionListener(this);
		actionButtons.add(OKButton);
		options.add(actionButtons);

		add(options);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		pack();

		// Centre on screen
		Dimension dim = getToolkit().getScreenSize();
		Rectangle abounds = getBounds();
		setLocation((dim.width - abounds.width) / 2,
				(dim.height - abounds.height) / 2);
		setVisible(true);
	}

	public void actionPerformed(ActionEvent evt) {
		if (evt.getSource() == mmRadioButton
				|| evt.getSource() == inchesRadioButton) {
			double multFactor = 1.0;
			if (evt.getSource() == mmRadioButton) {
				multFactor = (units.equals("inches")) ? 25.4 : 1.0;
				units = "mm";
			}
			if (evt.getSource() == inchesRadioButton) {
				multFactor = (units.equals("mm")) ? 1.0 / 25.4 : 1.0;
				units = "inches";
			}

			// Values are stored at a higher precision than the display so as to
			// avoid rounding errors when switching
			// between unit types
			distancePerRev = multFactor * distancePerRev;
			defaultFeedRate = multFactor * defaultFeedRate;
			curveSectionLength = multFactor * curveSectionLength;
			jogIncrement = multFactor * jogIncrement;

			// Round to two decimal places for display
			watchFieldChanges = false;
			distancePerRevField.setValue((double) Math
					.round(distancePerRev * 100) / 100);
			defaultFeedRateField.setValue((double) Math
					.round(defaultFeedRate * 100) / 100);
			curveSectionLengthField.setValue((double) Math
					.round(curveSectionLength * 100) / 100);
			jogIncrementField
					.setValue((double) Math.round(jogIncrement * 100) / 100);
			watchFieldChanges = true;
		}

		// OK clicked, so save preferences
		if (evt.getSource() == OKButton) {
			p.setProperty("Port(name)", portNameField.getText());
			p.setProperty("XAxisAddress", Integer
					.toString(((Number) xAddressField.getValue()).intValue()));
			p.setProperty("XAxisSteps", Integer.toString(((Number) xStepsField
					.getValue()).intValue()));
			p.setProperty("XAxisMaxSpeed", Integer
					.toString(((Number) xMaxSpeedField.getValue()).intValue()));
			p.setProperty("YAxisAddress", Integer
					.toString(((Number) yAddressField.getValue()).intValue()));
			p.setProperty("YAxisSteps", Integer.toString(((Number) yStepsField
					.getValue()).intValue()));
			p.setProperty("YAxisMaxSpeed", Integer
					.toString(((Number) yMaxSpeedField.getValue()).intValue()));
			p.setProperty("ZAxisAddress", Integer
					.toString(((Number) zAddressField.getValue()).intValue()));
			p.setProperty("ZAxisSteps", Integer.toString(((Number) zStepsField
					.getValue()).intValue()));
			p.setProperty("ZAxisMaxSpeed", Integer
					.toString(((Number) zMaxSpeedField.getValue()).intValue()));
			p
					.setProperty("Units", mmRadioButton.isSelected() ? "mm"
							: "inches");
			p.setProperty("DistancePerRev", Double
					.toString(((Number) distancePerRevField.getValue())
							.doubleValue()));
			p.setProperty("DefaultFeedRate", Double
					.toString(((Number) defaultFeedRateField.getValue())
							.doubleValue()));
			p.setProperty("CurveSectionLength", Double
					.toString(((Number) curveSectionLengthField.getValue())
							.doubleValue()));
			p.setProperty("JogIncrement", Double
					.toString(((Number) jogIncrementField.getValue())
							.doubleValue()));
			p.save();
			try {
				parent.loadPreferences();
			} catch (Exception e) {
			}
		}
		if (evt.getSource() == OKButton || evt.getSource() == cancelButton) {
			dispose();
		}
	}

	public void insertUpdate(DocumentEvent evt) {
		fieldChanged(evt);
	}

	public void removeUpdate(DocumentEvent evt) {
		fieldChanged(evt);
	}

	public void changedUpdate(DocumentEvent evt) {
	}

	/*
	 * Value of field changed, so update internal variable
	 */
	private void fieldChanged(DocumentEvent evt) {
		if (evt.getDocument().getProperty("name") == null || !watchFieldChanges)
			return;
		try {
			if (evt.getDocument().getProperty("name").equals("distancePerRev"))
				distancePerRev = ((Number) distancePerRevField.getValue())
						.doubleValue();
			if (evt.getDocument().getProperty("name").equals("defaultFeedRate"))
				defaultFeedRate = ((Number) defaultFeedRateField.getValue())
						.doubleValue();
			if (evt.getDocument().getProperty("name").equals(
					"curveSectionLength"))
				curveSectionLength = ((Number) curveSectionLengthField
						.getValue()).doubleValue();
			if (evt.getDocument().getProperty("name").equals("jogIncrement"))
				jogIncrement = ((Number) jogIncrementField.getValue())
						.doubleValue();
		} catch (NumberFormatException e) {

		}
	}
}